
import ctypes

usb = ctypes.cdll.LoadLibrary('usb.dll')
usb.initialize()

class bootloader:

    def __init__(self):
        self.buffer = ctypes.c_buffer(64)
        self.READ_EEPROM = 0
        self.WRITE_EEPROM = 1
        self.READ_FLASH = 2
        self.WRITE_FLASH = 3
        self.ERASE_FLASH = 4
        self.START_USER = 5
        self.dev = usb.open_device(0x6666, 0x4321, 0)
        if self.dev==-1:
            print "Found device with vendor ID = 0x6666 and product ID = 0x4321, but could not open it.\n"
        elif self.dev==-2:
            print "Found device with vendor ID = 0x6666 and product ID = 0x4321, but too many devices already open.\n"
        elif self.dev==-3:
            print "No device found with vendor ID = 0x6666 and product ID = 0x4321.\n"
        else:
            ret = usb.control_transfer(self.dev, 0x00, 0x09, 1, 0, 0, self.buffer)
            if ret<0:
                print "Could not send SET_CONFIGURATION(1) standard request.\n"

    def close(self):
        usb.close_device(self.dev)

    def read_eeprom(self, address, num_bytes):
        ret = usb.control_transfer(self.dev, 0xC0, self.READ_EEPROM, 0, address, num_bytes, self.buffer)
        if ret<0:
            print "Unable to send READ_EEPROM vendor request.\n"
        else:
            bytes = []
            for i in range(ret):
                bytes.append(ord(self.buffer.raw[i]))
            return bytes

    def write_eeprom(self, address, bytes):
        for i in range(len(bytes)):
            self.buffer[i] = chr(bytes[i])
        ret = usb.control_transfer(self.dev, 0x40, self.WRITE_EEPROM, 0, address, len(bytes), self.buffer)
        if ret<0:
            print "Unable to send WRITE_EEPROM vendor request.\n"

    def read_flash(self, address, num_bytes):
        ret = usb.control_transfer(self.dev, 0xC0, self.READ_FLASH, address>>16, address&0xFFFF, num_bytes, self.buffer)
        if ret<0:
            print "Unable to send READ_FLASH vendor request.\n"
        else:
            bytes = []
            for i in range(ret):
                bytes.append(ord(self.buffer.raw[i]))
            return bytes

    def write_flash(self, address, bytes):
        for i in range(len(bytes)):
            self.buffer[i] = chr(bytes[i])
        ret = usb.control_transfer(self.dev, 0x40, self.WRITE_FLASH, address>>16, address&0xFFFF, len(bytes), self.buffer)
        if ret<0:
            print "Unable to send WRITE_FLASH vendor request.\n" 

    def erase_flash(self, address):
        ret = usb.control_transfer(self.dev, 0x40, self.ERASE_FLASH, address>>16, address&0xFFFF, 0, self.buffer)
        if ret<0:
            print "Unable to send ERASE_FLASH vendor request.\n"

    def start_user(self):
        ret = usb.control_transfer(self.dev, 0x40, self.START_USER, 0, 0, 0, self.buffer)
        if ret<0:
            print "Unable to send START_USER vendor request.\n"
